# The LoRa daemon: `lorad`

The _LoRa daemon_ is a user space daemon that uses the _LoRa API_ to
provide a software abstraction for other process to the _LoRa devices_,
including, but not limited to, demodulators, transceivers and sniffers.

## Build

If you get the sources from `git` the `protos/` directory should contains
`proto` files. But, in doubts, you can update the submodules with:

```bash
git submodule update --init --recursive
```

Then, the build requires a configuration file. You can create one by yourself
or create a default configuration file with:

```bash
make defconfig
```

The `.config` file may be edited later. All available options are described in
the `defconfig` file.

The build is platform dependent and should be selected in the `.config` file
by setting the `CONFIG_PLATFORM` variable to one of the following:
`fevo`, `ibts` (default), `wiis`, `wifc` or `wsta`.

Finally, just run:

```bash
make
```

## Control channel

The _LoRa daemon_ has its own control channel as described in the _LoRa API_.

The endpoint for the control channel can be configured at runtime. Check the
`common/lorad.json` example configuration file.

The `lorad` control channel support these services:
 - Verbosity management: `"set_verbosity"`
 - Request for version: `"get_version"`
 - Frequency scan management: `"scan_status"`, `"scan_start"`, `"scan_stop"`
 - Request for statistics: `"get_statistics"`

### Available statistics

Statistics available on `lorad` are the following:

| Name | Type | Mandatory |
|--|--|--|
| uplink_received | unsigned int | yes |
| uplink_invalid | unsigned int | yes |
| uplink_valid | unsigned int | yes |
| uplink_bytes_received | unsigned int | yes |
| downlink_requested | unsigned int | yes |
| downlink_scheduled | unsigned int | yes |
| downlink_transmitted | unsigned int | yes |
| downlink_bytes_transmitted | unsigned int | yes |
| radio_temperature | float | no |
| gps_latitude | float | no |
| gps_longitude | float | no |
| gps_altitude | int | no |
| gps_lost_pulse | unsigned int | no |
| version_fpga | unsigned int | no |
| version_dsp | unsigned int | no |
| version_hal | string | no |

## Tools

In order to use the provided tools you have to build and install the
`lora-api.zip` python package. This package is built by default. It can also
be built separately with:

```bash
make lora-api.zip
```

Then you have to install it into your python search path. You can skip this
step by setting the `PYTHONPATH` environment variable:

```bash
export PYTHONPATH=lora-api.zip
```

The full usage of each tool can be printed with the `-h, --help` arguments.

### The LoRa receiver: `lorarecv`

The _LoRa receiver_ display every messages from the configured uplink channels.

```bash
./tools/lorarecv "tcp://198.51.100.42:1111"
```

### The LoRa sender: `lorasend`

The _LoRa sender_ push messages to the configured downlink channel.

```bash
./tools/lorasend "tcp://198.51.100.42:2222" --lora
```

### The LoRa controller: `loractrl`

The _LoRa controller_ send requests to the configured control channel.

```bash
./tools/loractrl "tcp://198.51.100.42:3333" --get-version
```

## License

This work is released under the [3-Clause BSD License](LICENSE.md).
