// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _COMMON_API_H_
#define _COMMON_API_H_

#include <pthread.h>
#include <stdint.h>

#include "protos/downlink-container.pb-c.h"
#include "protos/uplink-container.pb-c.h"

#include "core.h"

struct api {
	void *zmq_context;
	void *zmq_uplink;
	void *zmq_downlink;
	void *zmq_control;
	pthread_t rpc_thread;
	uint32_t message_id;
};

struct single_scan {
	float rssi;
	int32_t frequency_offset;
	uint32_t duration;
};

struct api *api_init(void);
int api_start(struct core *core);
int api_running(struct core *core);
int api_join(struct core *core);
void api_shutdown(struct core *core);
void api_free(struct core *core);

void api_free_lora(Proto__Lora__Uplink *message);
void api_free_gfsk(Proto__Gfsk__Uplink *message);

int api_send_lora(struct core *core, Proto__Lora__Uplink *message);
int api_send_gfsk(struct core *core, Proto__Gfsk__Uplink *message);

int api_send_event_system(struct core *core, Proto__Event__System__Event event);
int api_send_event_transmission(struct core *core,
				uint32_t message_id, uint32_t transmission_id,
				Proto__Event__Transmission__Status status);

int api_send_scan(struct core *core, uint32_t receiver_id, uint64_t sample_duration_us, uint32_t base_frequency,
		  uint32_t frequency_step, Proto__Time__Time *start_time, size_t n_single_scans,
		  struct single_scan *single_scans);

Proto__Downlink__Container *api_recv(struct core *core);

#endif
