// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _COMMON_CLOCK_H_
#define _COMMON_CLOCK_H_

#include <pthread.h>
#include <stdint.h>
#include <time.h>

#include "core.h"

struct clock_core;

struct clock_core *clock_core_init(void);
int clock_core_start(struct core *core);
int clock_core_join(struct core *core);
void clock_core_shutdown(struct core *core);
void clock_core_free(struct core *core);

struct clock_sync {
	int sync;
	uint32_t counter;
	struct timespec time;
	int32_t sum_drift;
};

struct clock {
	struct clock_core *core;

	int running;
	pthread_t valid_thread;

	int pps_fd;
	int pps_enabled;
	struct timespec pps_time;
	time_t tai_offset;
};

struct clock *clock_init(void);
int clock_start(struct core *core);
int clock_join(struct core *core);
void clock_shutdown(struct core *core);
void clock_free(struct core *core);

void clock_convert_tai_to_gps(struct timespec *tai, struct timespec *gps);
void clock_convert_gps_to_tai(struct timespec *gps, struct timespec *tai);
int clock_convert_tai_to_utc(time_t tai_offset, struct timespec *tai, struct timespec *utc);

int clock_get_monotonic_time(struct timespec *tp);
int clock_get_utc_time(struct timespec *tp);
int clock_get_tai_time(time_t tai_offset, struct timespec *tp);
int clock_get_gps_time(time_t tai_offset, struct timespec *tp);

#endif
