// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _CORE_H_
#define _CORE_H_

#include <pthread.h>
#include <stdint.h>

#include "utils/stringlist.h"

struct config;
struct api;
struct clock;
struct fscan;
struct statistics;
struct temperature;

struct core {
	int started;
	int running;

	int gps_enabled;
	int gps_fd;
	char gps_device[64];

	pthread_t uplink_thread;
	pthread_t downlink_thread;
	pthread_t jit_thread;
	pthread_t beacon_thread;
	pthread_t gps_thread;
	pthread_t temperature_thread;

	struct config *config;
	struct api *api;
	struct clock *clock;
	struct fscan *fscan;
	struct statistics *statistics;
	struct temperature *temperature;
};

struct core *core_init(struct stringlist *filenames, char *gps_device, uint32_t serial_number);
int core_start(struct core *core);
int core_join(struct core *core);
void core_shutdown(struct core *core);
void core_free(struct core *core);

void fill_version_statistics(struct statistics *statistics);

#endif
