// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _COMMON_FSCAN_H_
#define _COMMON_FSCAN_H_

#include <pthread.h>
#include <stdbool.h>
#include <stdint.h>
#include <time.h>

#include "core.h"

struct fscan_core;

struct sniffer_caps {
	size_t sniffer_nb;
};

struct sniffer {
	struct core *core;
	int running;
	pthread_t thread;
	struct {
		uint32_t receiver_id;
		size_t n_frequencies;
		uint32_t *frequencies;
		bool infinite_loop;
		uint32_t samples;
	} parameters;
};

struct fscan_core *fscan_core_init(void);
int fscan_core_start(uint32_t receiver_id);
void fscan_core_shutdown(uint32_t receiver_id);
void fscan_core_free(struct core *core);

struct fscan {
	struct fscan_core *core;

	struct sniffer_caps sniffer_caps;
	struct sniffer *sniffers;
};

struct fscan_receivers {
	uint32_t id;
	bool busy;
};

struct fscan *fscan_init(void);
int fscan_start(struct core *core);
int fscan_join(struct core *core);
void fscan_shutdown(struct core *core);
void fscan_free(struct core *core);

int fscan_get_scan_status(struct core *core, size_t *n_receivers, struct fscan_receivers **fscan_receivers);
int fscan_start_scan(struct core *core, uint32_t receiver_id, size_t n_frequencies,
		     uint32_t *frequencies, bool has_samples, uint32_t samples);
int fscan_stop_scan(struct core *core, uint32_t receiver_id);

#endif
