// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "temperature.h"
#include "core_temperature.h"

#include <stdlib.h>

#include "utils/clock.h"
#include "utils/logger.h"
#include "utils/memory.h"

static void *temperature_thread(void *arg)
{
	struct core *core = arg;

	while (core->temperature->running) {
		if (check_temperature(core) < 0) {
			warning("Maximum temperature reached");
			core->temperature->cooldown = true;
			core_shutdown(core);
		}
		sleep_sec(60, core->temperature->running);
	}

	pthread_exit(NULL);
}

struct temperature *temperature_init(void)
{
	struct temperature *temperature = NULL;

	if ((temperature = zalloc(sizeof(*temperature))) == NULL) {
		eerror("Failed to allocate temperature");
		return NULL;

	} else if ((temperature->core = temperature_core_init()) == NULL) {
		return NULL;
	}

	return temperature;
}

int temperature_start(struct core *core)
{
	core->temperature->running = 1;

	if (pthread_create(&core->temperature->temperature_thread, NULL, temperature_thread, core) != 0) {
		error("Failed to start temperature validation thread");
		return -1;
	}

	return 0;
}

int temperature_join(struct core *core)
{
	if (pthread_join(core->temperature->temperature_thread, NULL) != 0) {
		error("Failed to join temperature validation thread");
		return -1;
	}

	return 0;
}

void temperature_shutdown(struct core *core)
{
	if (core) {
		if (core->temperature) {
			core->temperature->running = 0;
		}
	}
}

void temperature_free(struct core *core)
{
	if (core) {
		temperature_core_free(core);
		free(core->temperature);
	}
}
