// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _CORE_CLOCK_H_
#define _CORE_CLOCK_H_

#include <stdint.h>
#include <sys/types.h>

#include "lib/sx1301ar/gps.h"
#include "lib/sx1301ar/hal.h"

#include "common/clock.h"

struct clock_core {
	int running;
	pthread_t sync_thread;
	pthread_mutex_t mutex;

	struct clock_board {
		struct clock_sync monotonic;
		struct clock_sync utc;
		struct clock_sync tai;

		sx1301ar_tref_t tref;
	} boards[SX1301AR_MAX_BOARD_NB];

	int synchronized;
};

int clock_get_current_counter(struct core *core, size_t nb_board, uint32_t *counter);
int clock_convert_counter(struct core *core, size_t from_board, size_t to_board, uint32_t from_counter, uint32_t *to_counter);

int clock_get_utc_time_from_counter(struct core *core, size_t nb_board, uint32_t counter, struct timespec *tp);
int clock_get_tai_time_from_counter(struct core *core, size_t nb_board, uint32_t counter, struct timespec *tp);
int clock_get_gps_time_from_counter(struct core *core, size_t nb_board, uint32_t counter, struct timespec *tp);

int clock_get_counter_from_utc_time(struct core *core, size_t nb_board, struct timespec *tp, uint32_t *counter);
int clock_get_counter_from_tai_time(struct core *core, size_t nb_board, struct timespec *tp, uint32_t *counter);
int clock_get_counter_from_gps_time(struct core *core, size_t nb_board, struct timespec *tp, uint32_t *counter);

#endif
