// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include <sys/types.h>

#include "utils/logger.h"
#include "utils/memory.h"

#include "common/statistics.h"
#include "common/temperature.h"
#include "config.h"

#define T_CODE_MAX 135 /* Approximately 85°C */

struct temperature_core *temperature_core_init(void)
{
	struct temperature_core *temperature_core = NULL;

	if ((temperature_core = zalloc(sizeof(struct temperature_core))) == NULL) {
		eerror("Failed to allocate temperature core");
		return NULL;
	}

	return temperature_core;
}

void temperature_core_free(struct core *core)
{
	if (core && core->temperature) {
		free(core->temperature->core);
	}
}

int check_temperature(struct core *core)
{
	int16_t t_celsius;
	uint8_t t_code;
	size_t board;
	int32_t sum_temperature = 0;

	/* To get ride of calibration issue, only the temperature code is used.
	 *
	 * The targeted temperature is 85°C which is approximately 135 in
	 * temperature code.
	 *
	 * On first start the AD9361 seems to return 202 as temperature code.
	 * So temperature codes above 200 are skipped.
	 */

	for (board = 0; board < core->config->core->nb_boards; board++) {
		if (sx1301ar_get_temperature(board, &t_code, &t_celsius) < 0) {
			error("Board #%zu: Failed to read temperature", board);
			return -1;
		} else if (t_code > T_CODE_MAX && t_code < 200) {
			error("Board #%zu: Too high temperature: code %u (%i C) > code %u",
			      board, t_code, t_celsius, T_CODE_MAX);
			return -1;
		}

		debug("Board #%zu: temperature: code %u (%i C) (code max %u)",
		      board, t_code, t_celsius, T_CODE_MAX);

		sum_temperature += t_celsius;
	}

	core->statistics->radio_temperature = (float)sum_temperature / core->config->core->nb_boards;
	core->statistics->has_radio_temperature = 1;

	debug("Mean temperature on %zu boards: %.1f C",
	      core->config->core->nb_boards, core->statistics->radio_temperature);

	return 0;
}
