// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _DRIVERS_SPI_H_
#define _DRIVERS_SPI_H_

#include <stddef.h>
#include <stdint.h>

#include "gpio.h"

// Mode configuration.
#define SPI_MODE_BIT 0x0001
#define SPI_CPHA 0x0002
#define SPI_CPOL 0x0004
#define SPI_MODE_0 (SPI_MODE_BIT | 0 | 0)
#define SPI_MODE_1 (SPI_MODE_BIT | SPI_CPHA | 0)
#define SPI_MODE_2 (SPI_MODE_BIT | 0 | SPI_CPOL)
#define SPI_MODE_3 (SPI_MODE_BIT | SPI_CPHA | SPI_CPOL)
#define SPI_MODE_MASK SPI_MODE_3

// Bytes ordering configuration.
#define SPI_BYTES_ORDER_BIT 0x0008
#define SPI_MSB_FIRST (SPI_BYTES_ORDER_BIT | 0)
#define SPI_LSB_FIRST (SPI_BYTES_ORDER_BIT | 0x0010)
#define SPI_BYTES_ORDER_MASK SPI_LSB_FIRST

// Bits per word configuration.
#define SPI_WORD_BIT 0x0020
#define SPI_WORD_8BITS (SPI_WORD_BIT | 0x0040)
#define SPI_WORD_16BITS (SPI_WORD_BIT | 0x0080)
#define SPI_WORD_32BITS (SPI_WORD_BIT | 0x00C0)
#define SPI_WORD_MASK SPI_WORD_32BITS

// Change chip select behavior for the message.
#define SPI_CS_CHANGE 0x0100

// This message is a header.
#define SPI_MSG_HEADER 0x0200

struct spi_msg {
	const void *mosi;
	void *miso;
	size_t len;

	uint32_t speed_hz;
	uint16_t mode;
	uint16_t delay_us;
};

struct spi_device {
	void (*close)(struct spi_device *dev);

	int (*get_speed)(struct spi_device *dev, uint32_t *speed_hz);
	int (*set_speed)(struct spi_device *dev, uint32_t speed_hz);

	int (*get_mode)(struct spi_device *dev, uint16_t *mode);
	int (*set_mode)(struct spi_device *dev, uint16_t mode);

	int (*msg)(struct spi_device *dev, struct spi_msg *msg, size_t len);
};

struct spi_device *spi_open(const char *path, struct gpio_device *csn);

#endif
