// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _CALIB_H_
#define _CALIB_H_

#include <stdint.h>

#include "utils/memory.h"

#define CALIB_DIG_MASK 0x03
#define CALIB_DIG_SHIFT 0
#define CALIB_PA_MASK 0x0C
#define CALIB_PA_SHIFT 2
#define CALIB_MIX_MASK 0xF0
#define CALIB_MIX_SHIFT 4

#define CALIB_GAIN(name, gain) (gain & CALIB_##name##_MASK) >> CALIB_##name##_SHIFT

struct calib_tx_lut {
	uint8_t dig_pa_mix_gain;
	int8_t rf_power;
};

#define CALIB_MANUFACTURER_KERLINK 0x00

#define CALIB_BOARD_TYPE_WSTA_868MHZ 0x00
#define CALIB_BOARD_TYPE_WSTA_868MHZ_27DBM 0x01
#define CALIB_BOARD_TYPE_WSTA_915MHZ_27DBM 0x02
#define CALIB_BOARD_TYPE_WSTA_923MHZ_27DBM 0x03
#define CALIB_BOARD_TYPE_WSTA_433MHZ 0x04
#define CALIB_BOARD_TYPE_WSTA_923MHZ_27DBM_REVB 0x05
#define CALIB_BOARD_TYPE_WSTA_868MHZ_27DBM_REVB 0x06
#define CALIB_BOARD_TYPE_WSTA_868MHZ_27DBM_REVC 0x07
#define CALIB_BOARD_TYPE_WSTA_915MHZ_27DBM_REVB 0x08
#define CALIB_BOARD_TYPE_WSTA_923MHZ_27DBM_REVC 0x09
#define CALIB_BOARD_TYPE_WIFC_868MHZ 0x10
#define CALIB_BOARD_TYPE_WIFC_915MHZ 0x11
#define CALIB_BOARD_TYPE_WIFC_923MHZ 0x12
#define CALIB_BOARD_TYPE_WIIS_923MHZ 0x54
#define CALIB_BOARD_TYPE_WIIS_915MHZ 0x55
#define CALIB_BOARD_TYPE_WIIS_868MHZ 0x56
#define CALIB_BOARD_TYPE_FEVO_868MHZ 0x64
#define CALIB_BOARD_TYPE_FEVO_915MHZ 0x65
#define CALIB_BOARD_TYPE_FEVO_923MHZ 0x66

#define CALIB_RADIO_0 0
#define CALIB_RADIO_1 1
#define CALIB_SNIFFER 2

struct calib {
	uint8_t manufacturer;
	uint8_t board_type;
	uint8_t hardware_version;
	uint8_t serial_number[3];
	uint8_t reserved[10];
	int32_t frequency_offset;
	float rssi_offset[3];
	struct calib_tx_lut tx_lut[16];
};

#define CALIB_SIZE_HEADER sizeofs(struct calib, manufacturer) +         \
			      sizeofs(struct calib, board_type) +       \
			      sizeofs(struct calib, hardware_version) + \
			      sizeofs(struct calib, serial_number)
#define CALIB_SIZE_OFFSET sizeofs(struct calib, frequency_offset) + \
			      sizeofs(struct calib, rssi_offset)
#define CALIB_SIZE_TX_LUT sizeofs(struct calib, tx_lut)

struct calib *calib_read(void);

const char *calib_manufacturer_name(uint8_t manufacturer);
const char *calib_board_name(uint8_t board_type);

#endif
