// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "clock.h"

#include "utils/logger.h"

#define EXP9 1000000000
#define EXP6 1000000
#define EXP3 1000

#define time_add(tp, value, exp)                                               \
	{                                                                      \
		while (value > exp) {                                          \
			value -= exp;                                          \
			(tp)->tv_sec++;                                        \
		}                                                              \
                                                                               \
		(tp)->tv_nsec += value * (EXP9 / exp);                         \
                                                                               \
		while ((tp)->tv_nsec > EXP9) {                                 \
			(tp)->tv_nsec -= EXP9;                                 \
			(tp)->tv_sec++;                                        \
		}                                                              \
	}

void time_add_ns(struct timespec *tp, size_t t) time_add(tp, t, EXP9);
void time_add_us(struct timespec *tp, size_t t) time_add(tp, t, EXP6);
void time_add_ms(struct timespec *tp, size_t t) time_add(tp, t, EXP3);

#define time_diff(from, to, exp)                                               \
	{                                                                      \
		ssize_t diff;                                                  \
                                                                               \
		diff = ((to)->tv_sec - (from)->tv_sec) * exp;                  \
		diff += ((to)->tv_nsec - (from)->tv_nsec) / (EXP9 / exp);      \
                                                                               \
		return diff;                                                   \
	}

ssize_t time_diff_ns(struct timespec *from, struct timespec *to)
    time_diff(from, to, EXP9);
ssize_t time_diff_us(struct timespec *from, struct timespec *to)
    time_diff(from, to, EXP6);
ssize_t time_diff_ms(struct timespec *from, struct timespec *to)
    time_diff(from, to, EXP3);

#define sleep(value, exp)                                                      \
	{                                                                      \
		struct timespec tp;                                            \
		int err;                                                       \
                                                                               \
		if (clock_gettime(CLOCK_MONOTONIC, &tp) < 0) {                 \
			eerror("Failed to get time");                          \
			return -1;                                             \
		}                                                              \
                                                                               \
		time_add(&tp, value, exp);                                     \
                                                                               \
		err = clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &tp,     \
				      NULL);                                   \
                                                                               \
		if (err != 0 && err != EINTR) {                                \
			error("Failed to sleep");                              \
			return -1;                                             \
		}                                                              \
                                                                               \
		return 0;                                                      \
	}

int sleep_ns(size_t t) sleep(t, EXP9);
int sleep_us(size_t t) sleep(t, EXP6);
int sleep_ms(size_t t) sleep(t, EXP3);
