// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _UTILS_JSON_H_
#define _UTILS_JSON_H_

#include <stdio.h>

#include "third-party/parson.h"

#include "utils/logger.h"

#define JSON_BOOLEAN(object, name, value)                                      \
	({                                                                     \
		JSON_Value *_v = NULL;                                         \
		int ret = 0;                                                   \
		if ((_v = json_object_dotget_value(object, name)) != NULL) {   \
			if (json_value_get_type(_v) == JSONBoolean) {          \
				value = json_value_get_boolean(_v);            \
				debug("Value \"" name "\" configured to: %s",  \
				      (value) ? "true" : "false");             \
				ret = 1;                                       \
			} else {                                               \
				warning("Wrong data type for \"" name "\"");   \
			}                                                      \
		}                                                              \
		ret;                                                           \
	})

#define JSON_NUMBER(object, name, value)                                       \
	({                                                                     \
		JSON_Value *_v = NULL;                                         \
		int ret = 0;                                                   \
		if ((_v = json_object_dotget_value(object, name)) != NULL) {   \
			if (json_value_get_type(_v) == JSONNumber) {           \
				value = json_value_get_number(_v);             \
				debug("Value \"" name                          \
				      "\" configured to: %.10g",               \
				      (double)value);                          \
				ret = 1;                                       \
			} else {                                               \
				warning("Wrong data type for \"" name "\"");   \
			}                                                      \
		}                                                              \
		ret;                                                           \
	})

#define JSON_STRING(object, name, value)                                       \
	({                                                                     \
		JSON_Value *_v = NULL;                                         \
		int ret = 0;                                                   \
		if ((_v = json_object_dotget_value(object, name)) != NULL) {   \
			if (json_value_get_type(_v) == JSONString) {           \
				value = json_value_get_string(_v);             \
				debug("Value \"" name                          \
				      "\" configured to: \"%s\"",              \
				      value);                                  \
				ret = 1;                                       \
			} else {                                               \
				warning("Wrong data type for \"" name "\"");   \
			}                                                      \
		}                                                              \
		ret;                                                           \
	})

#define JSON_HEX(object, name, value)                                          \
	({                                                                     \
		JSON_Value *_v = NULL;                                         \
		int ret = 0;                                                   \
		if ((_v = json_object_dotget_value(object, name)) != NULL) {   \
			if (json_value_get_type(_v) == JSONNumber) {           \
				value = json_value_get_number(_v);             \
				debug("Value \"" name "\" configured to: %X",  \
				      value);                                  \
				ret = 1;                                       \
			} else if (json_value_get_type(_v) == JSONString) {    \
				if (sscanf(json_value_get_string(_v), "%x",    \
					   &value) == 1) {                     \
					debug("Value \"" name                  \
					      "\" configured to: %X",          \
					      value);                          \
					ret = 1;                               \
				} else {                                       \
					warning("Wrong format for \"" name     \
						"\"");                         \
				}                                              \
			} else {                                               \
				warning("Wrong data type for \"" name "\"");   \
			}                                                      \
		}                                                              \
		ret;                                                           \
	})

#endif
