// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _UTILS_LOGGER_H_
#define _UTILS_LOGGER_H_

#include <errno.h>
#include <string.h>
#include <syslog.h>

#define logger(p, fmt, args...) syslog(p, "<%u> " fmt, p, ##args)

#ifdef VERBOSE_LOGGER
#undef logger
#define logger(p, fmt, args...)                                                \
	syslog(p, "<%u> [%s@%s+%u] " fmt, p, __func__, __FILE__, __LINE__,     \
	       ##args)

#endif

#define setlogpriority(p)                                                      \
	setlogmask(LOG_UPTO(                                                   \
	    (p < LOG_EMERG) ? LOG_EMERG : (p > LOG_DEBUG) ? LOG_DEBUG : p))

#define emergency(fmt, args...) logger(LOG_EMERG, fmt, ##args)
#define alert(fmt, args...) logger(LOG_ALERT, fmt, ##args)
#define critical(fmt, args...) logger(LOG_CRIT, fmt, ##args)
#define error(fmt, args...) logger(LOG_ERR, fmt, ##args)
#define warning(fmt, args...) logger(LOG_WARNING, fmt, ##args)
#define notice(fmt, args...) logger(LOG_NOTICE, fmt, ##args)
#define info(fmt, args...) logger(LOG_INFO, fmt, ##args)
#define debug(fmt, args...) logger(LOG_DEBUG, fmt, ##args)

#define eemergency(fmt, args...) emergency(fmt ": %s", ##args, strerror(errno))
#define ealert(fmt, args...) alert(fmt ": %s", ##args, strerror(errno))
#define ecritical(fmt, args...) critical(fmt ": %s", ##args, strerror(errno))
#define eerror(fmt, args...) error(fmt ": %s", ##args, strerror(errno))
#define ewarning(fmt, args...) warning(fmt ": %s", ##args, strerror(errno))
#define enotice(fmt, args...) notice(fmt ": %s", ##args, strerror(errno))
#define einfo(fmt, args...) info(fmt ": %s", ##args, strerror(errno))
#define edebug(fmt, args...) debug(fmt ": %s", ##args, strerror(errno))

void dump4(const void *buf, size_t len, const char *prefix, int dump);
#define dump3(buf, len, prefix) dump4(buf, len, prefix, 32)
#define dump2(buf, len) dump3(buf, len, "dump")

#define _dump_suffix(buf, len, prefix, dump, suffix, ...) suffix
#define dump_suffix(args...) _dump_suffix(args, 4, 3, 2)
#define __dump(suffix, args...) dump##suffix(args)
#define _dump(suffix, args...) __dump(suffix, args)
#define dump(args...) _dump(dump_suffix(args), args)

#endif
