// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "spi.h"

#include <fcntl.h>
#include <stdint.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <unistd.h>

#include "utils/logger.h"

int spi_init(const char *spidev, int speed, int mode, int bits, int lsb)
{
	int spi;

	if ((spi = open(spidev, O_RDWR)) < 0) {
		eerror("Failed to open spi");
		return -1;

	} else if (ioctl(spi, SPI_IOC_WR_MAX_SPEED_HZ, &speed) < 0) {
		eerror("Failed to configure spi speed");
		close(spi);
		return -1;

	} else if (ioctl(spi, SPI_IOC_WR_MODE, &mode) < 0) {
		eerror("Failed to configure spi mode");
		close(spi);
		return -1;

	} else if (ioctl(spi, SPI_IOC_WR_BITS_PER_WORD, &bits) < 0) {
		eerror("Failed to configure spi bits per word");
		close(spi);
		return -1;

	} else if (ioctl(spi, SPI_IOC_WR_LSB_FIRST, &lsb) < 0) {
		eerror("Failed to configure spi lsb first");
		close(spi);
		return -1;
	}

	return spi;
}

int spi_msg(int spi, const void *mosi, void *miso, size_t len)
{
	struct spi_ioc_transfer msg;

	memset(&msg, 0, sizeof(msg));
	msg.tx_buf = (uintptr_t)mosi;
	msg.rx_buf = (uintptr_t)miso;
	msg.len = len;

	if (ioctl(spi, SPI_IOC_MESSAGE(1), &msg) < (int)len) {
		eerror("Failed to send ioctl message");
		return -1;
	}

	return 0;
}
